@@ -3,6 +3,7 @@ package proc
 import (
 	"bytes"
 	"cmp"
+	"context"
 	"debug/dwarf"
 	"debug/elf"
 	"debug/macho"
@@ -34,10 +35,10 @@ import (
 	"github.com/go-delve/delve/pkg/dwarf/reader"
 	"github.com/go-delve/delve/pkg/goversion"
 	"github.com/go-delve/delve/pkg/internal/gosym"
+	"github.com/go-delve/delve/pkg/internal/lru"
 	"github.com/go-delve/delve/pkg/logflags"
 	"github.com/go-delve/delve/pkg/proc/debuginfod"
 	"github.com/go-delve/delve/pkg/proc/evalop"
-	"github.com/hashicorp/golang-lru/simplelru"
 )
 
 const (
@@ -116,6 +117,10 @@ type BinaryInfo struct {
 	debugPinnerFn *Function
 	logger        logflags.Logger
 	eventsFn      func(*Event)
+
+	cancelDownloadsMu sync.Mutex
+	cancelDownloads   func()
+	downloadsCtx      context.Context
 }
 
 var (
@@ -287,7 +292,7 @@ func FindFileLocation(p Process, filename string, lineno int) ([]uint64, error)
 		}
 	}
 
-	sort.Slice(selectedPCs, func(i, j int) bool { return selectedPCs[i] < selectedPCs[j] })
+	slices.Sort(selectedPCs)
 
 	return selectedPCs, nil
 }
@@ -369,7 +374,7 @@ func FindFunctionLocation(p Process, funcName string, lineOffset int) ([]uint64,
 			return nil, &ErrFunctionNotFound{funcName}
 		}
 	}
-	sort.Slice(r, func(i, j int) bool { return r[i] < r[j] })
+	slices.Sort(r)
 	return r, nil
 }
 
@@ -400,7 +405,7 @@ func FirstPCAfterPrologue(p Process, fn *Function, sameline bool) (uint64, error
 		// Look for the first instruction with the stmt flag set, so that setting a
 		// breakpoint with file:line and with the function name always result on
 		// the same instruction being selected.
-		if pc2, _, _, ok := fn.cu.lineInfo.FirstStmtForLine(fn.Entry, fn.End); ok {
+		if pc2, _, _, ok := fn.cu.lineInfo.FirstStmt(fn.Entry, fn.End); ok {
 			return pc2, nil
 		}
 	}
@@ -527,19 +532,18 @@ type Function struct {
 	// extraCache contains information about this function that is only needed for
 	// some operations and is expensive to compute or store for every function.
 	extraCache *functionExtra
+	// rangeBodiesCache caches the list of range-over-func body closures for
+	// this function. Only one of extraCache.rangeParent and rangeBodiesCache
+	// should be set at any given time.
+	rangeBodiesCache []*Function
 }
 
 type functionExtra struct {
 	// closureStructType is the cached struct type for closures for this function
 	closureStructType *godwarf.StructType
-
 	// rangeParent is set when this function is a range-over-func body closure
 	// and points to the function that the closure was generated from.
 	rangeParent *Function
-	// rangeBodies is the list of range-over-func body closures for this
-	// function. Only one between rangeParent and rangeBodies should be set at
-	// any given time.
-	rangeBodies []*Function
 }
 
 // instRange returns the indexes in fn.Name of the type parameter
@@ -574,10 +578,7 @@ func (fn *Function) PackageName() string {
 }
 
 func packageName(name string) string {
-	pathend := strings.LastIndex(name, "/")
-	if pathend < 0 {
-		pathend = 0
-	}
+	pathend := max(strings.LastIndex(name, "/"), 0)
 
 	if i := strings.Index(name[pathend:], "."); i != -1 {
 		return name[:pathend+i]
@@ -590,10 +591,7 @@ func packageName(name string) string {
 // Borrowed from $GOROOT/debug/gosym/symtab.go
 func (fn *Function) ReceiverName() string {
 	inst := fn.instRange()
-	pathend := strings.LastIndex(fn.Name[:inst[0]], "/")
-	if pathend < 0 {
-		pathend = 0
-	}
+	pathend := max(strings.LastIndex(fn.Name[:inst[0]], "/"), 0)
 	l := strings.Index(fn.Name[pathend:], ".")
 	if l == -1 {
 		return ""
@@ -718,6 +716,9 @@ func (fn *Function) rangeParentName() string {
 // extra loads information about fn that is expensive to compute and we
 // only need for a minority of the functions.
 func (fn *Function) extra(bi *BinaryInfo) *functionExtra {
+	if fn == nil {
+		return &functionExtra{}
+	}
 	if fn.extraCache != nil {
 		return fn.extraCache
 	}
@@ -770,17 +771,29 @@ func (fn *Function) extra(bi *BinaryInfo) *functionExtra {
 		fn.extraCache.rangeParent = bi.lookupOneFunc(rangeParentName)
 	}
 
-	// Find range-over-func bodies of this function
-	if fn.extraCache.rangeParent == nil {
-		for i := range bi.Functions {
-			fn2 := &bi.Functions[i]
-			if strings.HasPrefix(fn2.Name, fn.Name) && fn2.rangeParentName() == fn.Name {
-				fn.extraCache.rangeBodies = append(fn.extraCache.rangeBodies, fn2)
-			}
+	return fn.extraCache
+}
+
+// rangeBodies returns the list of range-over-func body closures for this
+// function.
+func (fn *Function) rangeBodies(bi *BinaryInfo) []*Function {
+	if fn == nil {
+		return nil
+	}
+	if fn.rangeBodiesCache != nil {
+		return fn.rangeBodiesCache
+	}
+	extra := fn.extra(bi)
+	if extra.rangeParent != nil {
+		return nil
+	}
+	for i := range bi.Functions {
+		fn2 := &bi.Functions[i]
+		if strings.HasPrefix(fn2.Name, fn.Name) && fn2.rangeParentName() == fn.Name {
+			fn.rangeBodiesCache = append(fn.rangeBodiesCache, fn2)
 		}
 	}
-
-	return fn.extraCache
+	return fn.rangeBodiesCache
 }
 
 type constantsMap map[dwarfRef]*constantType
@@ -1011,7 +1024,7 @@ type Image struct {
 
 	compileUnits []*compileUnit // compileUnits is sorted by increasing DWARF offset
 
-	dwarfTreeCache  *simplelru.LRU
+	dwarfTreeCache  *lru.Cache[dwarf.Offset, *godwarf.Tree]
 	workaroundCache map[dwarf.Offset]*godwarf.Tree
 
 	// runtimeTypeToDIE maps between the offset of a runtime._type in
@@ -1051,9 +1064,17 @@ func (bi *BinaryInfo) AddImage(path string, addr uint64) error {
 		}
 	}
 
+	if len(bi.Images) > 0 {
+		bi.cancelDownloadsMu.Lock()
+		if bi.cancelDownloads == nil {
+			bi.downloadsCtx, bi.cancelDownloads = context.WithCancel(context.Background())
+		}
+		bi.cancelDownloadsMu.Unlock()
+	}
+
 	// Actually add the image.
 	image := &Image{Path: path, addr: addr, typeCache: make(map[dwarf.Offset]godwarf.Type)}
-	image.dwarfTreeCache, _ = simplelru.NewLRU(dwarfTreeCacheSize, nil)
+	image.dwarfTreeCache = lru.NewCache[dwarf.Offset, *godwarf.Tree](dwarfTreeCacheSize)
 
 	// add Image regardless of error so that we don't attempt to re-add it every time we stop
 	image.index = len(bi.Images)
@@ -1066,6 +1087,21 @@ func (bi *BinaryInfo) AddImage(path string, addr uint64) error {
 	return err
 }
 
+// LoadImageBinaryInfoAgain loads the n-th image debug symbols if they weren't already loaded.
+func (bi *BinaryInfo) LoadImageBinaryInfoAgain(n int) error {
+	if n < 0 || n >= len(bi.Images) || bi.Images[n].loadErr == nil {
+		return nil
+	}
+	image := bi.Images[n]
+	image.loadErr = nil
+	err := loadBinaryInfo(bi, image, image.Path, image.addr)
+	if err != nil {
+		image.loadErr = err
+	}
+	bi.macOSDebugFrameBugWorkaround()
+	return err
+}
+
 // moduleDataToImage finds the image corresponding to the given module data object.
 func (bi *BinaryInfo) moduleDataToImage(md *ModuleData) *Image {
 	fn := bi.PCToFunc(md.text)
@@ -1151,7 +1187,7 @@ func (image *Image) Close() error {
 	return err2
 }
 
-func (image *Image) setLoadError(logger logflags.Logger, fmtstr string, args ...interface{}) {
+func (image *Image) setLoadError(logger logflags.Logger, fmtstr string, args ...any) {
 	image.loadErrMu.Lock()
 	image.loadErr = fmt.Errorf(fmtstr, args...)
 	image.loadErrMu.Unlock()
@@ -1170,7 +1206,7 @@ func (image *Image) getDwarfTree(off dwarf.Offset) (*godwarf.Tree, error) {
 		return image.workaroundCache[off], nil
 	}
 	if r, ok := image.dwarfTreeCache.Get(off); ok {
-		return r.(*godwarf.Tree), nil
+		return r, nil
 	}
 	r, err := godwarf.LoadTree(off, image.dwarf, image.StaticBase)
 	if err != nil {
@@ -1192,7 +1228,7 @@ func (bi *BinaryInfo) LoadImageFromData(dwdata *dwarf.Data, debugFrameBytes, deb
 	image.sepDebugCloser = (*nilCloser)(nil)
 	image.dwarf = dwdata
 	image.typeCache = make(map[dwarf.Offset]godwarf.Type)
-	image.dwarfTreeCache, _ = simplelru.NewLRU(dwarfTreeCacheSize, nil)
+	image.dwarfTreeCache = lru.NewCache[dwarf.Offset, *godwarf.Tree](dwarfTreeCacheSize)
 
 	if debugFrameBytes != nil {
 		bi.frameEntries, _ = frame.Parse(debugFrameBytes, frame.DwarfEndian(debugFrameBytes), 0, bi.Arch.PtrSize(), 0)
@@ -1572,7 +1608,7 @@ func (bi *BinaryInfo) openSeparateDebugInfo(image *Image, exe *elf.File, debugIn
 				})
 			}
 		}
-		debugFilePath, err = debuginfod.GetDebuginfo(notify, image.BuildID)
+		debugFilePath, err = debuginfod.GetDebuginfo(bi.downloadsCtx, notify, image.BuildID)
 		if err != nil {
 			return nil, nil, ErrNoDebugInfoFound
 		}
@@ -2541,7 +2577,7 @@ func (bi *BinaryInfo) loadDebugInfoMaps(image *Image, debugInfoBytes, debugLineB
 			}
 			lineInfoOffset, hasLineInfo := entry.Val(dwarf.AttrStmtList).(int64)
 			if hasLineInfo && lineInfoOffset >= 0 && lineInfoOffset < int64(len(debugLineBytes)) {
-				var logfn func(string, ...interface{})
+				var logfn func(string, ...any)
 				if logflags.DebugLineErrors() {
 					logfn = logflags.DebugLineLogger().Debugf
 				}
@@ -2985,10 +3021,7 @@ func (bi *BinaryInfo) expandPackagesInType(expr ast.Expr) {
 // elements of the path except the first one) like Go does in variable and
 // type names.
 func escapePackagePath(pkg string) string {
-	slash := strings.Index(pkg, "/")
-	if slash < 0 {
-		slash = 0
-	}
+	slash := max(strings.Index(pkg, "/"), 0)
 	return pkg[:slash] + strings.ReplaceAll(pkg[slash:], ".", "%2e")
 }
 
