@@ -48,6 +48,12 @@ func NewClientFromConn(conn net.Conn) *RPCClient {
 	return newFromRPCClient(jsonrpc.NewClient(conn))
 }
 
+func (c *RPCClient) GetVersion() *api.GetVersionOut {
+	out := new(api.GetVersionOut)
+	c.call("GetVersion", api.GetVersionIn{}, out)
+	return out
+}
+
 func (c *RPCClient) ProcessPid() int {
 	out := new(ProcessPidOut)
 	c.call("ProcessPid", ProcessPidIn{}, out)
@@ -693,17 +699,27 @@ func (c *RPCClient) GuessSubstitutePath() ([][2]string, error) {
 	return out.List, err
 }
 
+func (c *RPCClient) CancelDownloads() error {
+	out := &CancelDownloadsOut{}
+	return c.call("CancelDownloads", CancelDownloadsIn{}, out)
+}
+
+func (c *RPCClient) DownloadLibraryDebugInfo(n int) error {
+	out := DownloadLibraryDebugInfoOut{}
+	return c.call("DownloadLibraryDebugInfo", DownloadLibraryDebugInfoIn{n}, out)
+}
+
 func (c *RPCClient) call(method string, args, reply interface{}) error {
 	return c.client.Call("RPCServer."+method, args, reply)
 }
 
-func (c *RPCClient) callWhileDrainingEvents(method string, args, reply interface{}) error {
+func (c *RPCClient) callWhileDrainingEvents(method string, args, reply any) error {
 	done := c.drainEvents()
 	err := c.call(method, args, reply)
 	<-done
 	return err
 }
 
-func (c *RPCClient) CallAPI(method string, args, reply interface{}) error {
+func (c *RPCClient) CallAPI(method string, args, reply any) error {
 	return c.call(method, args, reply)
 }
